﻿using System;
using System.Collections.Generic;

using SFML.Window;
using SFML.Graphics;

namespace blendosvnvis
{
    class HourlySummary
    {
        int[] hourbreakdown;
        RectangleShape[] shapes;
        Text[] textTime;
        Text[] countTime;

        public HourlySummary(List<Checkin> checkins, Font font, uint monitorWidth)
        {
            const int BARHEIGHT = 18;

            Color barcolor = new Color(0, 70, 255);

            //do the time summaries.
            hourbreakdown = new int[24];
            shapes = new RectangleShape[hourbreakdown.Length];
            textTime = new Text[hourbreakdown.Length];
            countTime = new Text[hourbreakdown.Length];

            for (int i = 0; i < checkins.Count; i++)
            {
                hourbreakdown[checkins[i].date.Hour]++;
            }

            int highestCount = -1;
            for (int i = 0; i < hourbreakdown.Length; i++)
            {
                if (hourbreakdown[i] > highestCount)
                {
                    highestCount = hourbreakdown[i];
                }
            }

            float barWidth = (monitorWidth - 80) / (float)highestCount;


            for (int i = 0; i < shapes.Length; i++)
            {
                shapes[i] = new RectangleShape(new Vector2f(hourbreakdown[i] * barWidth, BARHEIGHT));
                shapes[i].FillColor = barcolor;
                shapes[i].Position = new Vector2f(12, 40 + (i * (BARHEIGHT + 2)));

                string timestr = string.Empty;

                if (i <= 0)
                {
                    timestr = "12 AM";
                }
                else if (i == 12)
                {
                    timestr = "12 PM";
                }
                else if (i >= 13)
                {
                    timestr = (i - 12).ToString() + " PM";
                }
                else
                {
                    timestr = i + " AM";
                }

                textTime[i] = new Text(timestr, font);
                textTime[i].CharacterSize = 16;
                textTime[i].Position = shapes[i].Position;
                textTime[i].Color = new Color(192, 192, 192);

                countTime[i] = new Text(hourbreakdown[i].ToString(), font);
                countTime[i].CharacterSize = 16;
                countTime[i].Position = shapes[i].Position + new Vector2f(shapes[i].Size.X + 5, 0);

                if (countTime[i].Position.X < shapes[i].Position.X + 45)
                {
                    countTime[i].Position = new Vector2f(shapes[i].Position.X + 45, countTime[i].Position.Y);
                }

                countTime[i].Color = barcolor;
            }
        }

        public void Draw(RenderWindow app)
        {
            for (int i = 0; i < shapes.Length; i++)
            {
                app.Draw(shapes[i]);
                app.Draw(textTime[i]);
                app.Draw(countTime[i]);
            }
        }
    }
}
