﻿using System;
using System.Collections.Generic;

using SFML.Graphics;
using SFML.Window;

namespace blendosvnvis
{
    class MonthSummary
    {
        public int month;
        public int year;
        public int count = 1;
        public RectangleShape shape;
        bool showYear;
        int index;

        Text countText;
        Text dateText;
        Text yearText;

        float delta;

        public void Initialize(int index, uint monitorWidth, uint monitorHeight, Font font, bool showYear, float barchunkHeight)
        {
            this.index = index;
            shape = new RectangleShape();
            shape.Size = new Vector2f(24, count * barchunkHeight);
            shape.Origin = new Vector2f(0, count * barchunkHeight);
            shape.FillColor = new Color(0, 128, 0);
            shape.Position = new Vector2f(32 + (index * (24 + 2)), monitorHeight - 24);

            countText = new Text(count.ToString(), font);
            countText.Position = shape.Position + new Vector2f(0, -shape.Size.Y - 16);
            countText.Color = new Color(0, 128, 0);
            countText.CharacterSize = 13;
            countText.Origin = new Vector2f(countText.GetLocalBounds().Width / 2, 0);

            dateText = new Text(GetMonthNameShort(month), font);
            dateText.CharacterSize = 17;
            dateText.Position = shape.Position + new Vector2f(0, -5);
            dateText.Color = new Color(192,192,192);
            dateText.Rotation = -90;
            

            this.showYear = showYear;
            if (showYear)
            {
                yearText = new Text(year.ToString(), font);
                yearText.CharacterSize = 20;
                yearText.Position = shape.Position + new Vector2f(0, -2);
                yearText.Color = new Color(0, 128, 0);
            }

            Reposition(0);
        }

        public void Reposition(float delta)
        {
            this.delta += delta;

            shape.Position = new Vector2f(32 + this.delta + (index * (24 + 2)), shape.Position.Y);
            countText.Position = shape.Position + new Vector2f(shape.Size.X/2, -shape.Size.Y - 16);
            dateText.Position = shape.Position + new Vector2f(0, -5);

            if (countText.Position.Y > shape.Position.Y - 60)
            {
                countText.Position = new Vector2f(countText.Position.X, shape.Position.Y - 55);
            }

            if (showYear)
                yearText.Position = shape.Position + new Vector2f(0, -2);
        }

        public void Reset()
        {
            this.delta = 0;
            Reposition(0);
        }
        
        public string GetMonthNameShort(int index)
        {
            switch (index)
            {
                case 1: return "Jan";
                case 2: return "Feb";
                case 3: return "Mar";
                case 4: return "Apr";
                case 5: return "May";
                case 6: return "Jun";
                case 7: return "Jul";
                case 8: return "Aug";
                case 9: return "Sep";
                case 10: return "Oct";
                case 11: return "Nov";
                case 12: return "Dec";
            }

            return "";
        }

        public void Draw(RenderWindow app)
        {
            app.Draw(shape);
            app.Draw(dateText);
            app.Draw(countText);

            if (showYear)
            {
                app.Draw(yearText);
            }
        }
    }
}
