﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Windows.Forms;

using SFML.Window;
using SFML.Graphics;

namespace blendosvnvis
{
    class Checkin
    {
        public int index;
        public DateTime date;
    }

    class Program
    {
        [STAThread]
        static void Main(string[] args)
        {
            Program program = new Program();
            program.Go();            
        }

        RenderWindow app;
        List<Checkin> checkins;
        List<MonthSummary> monthSummaries;
        HourlySummary hourlySummary;
        WeekSummary weekSummary;
        int drawmode;

        Text helptext;

        void ParseCheckins(string filepath)
        {
            string rawtext = GetFileContents(filepath);

            if (rawtext == null)
            {
                app.Close();
                return;
            }

            string[] rawLines = rawtext.Split(new string[] { "\r\n", "\n" }, StringSplitOptions.RemoveEmptyEntries);

            //Parse log file.
            for (int i = rawLines.Length - 1; i >= 0; i--)
            {
                Checkin newCheckin = new Checkin();

                if (rawLines[i].StartsWith("revision: ", true, System.Globalization.CultureInfo.InvariantCulture))
                {
                    string[] chunks = rawLines[i].Split(new char[] { ' ', '\t' }, StringSplitOptions.RemoveEmptyEntries);
                    newCheckin.index = int.Parse(chunks[1]);

                    int firstComma = rawLines[i + 2].IndexOf(',');
                    int secondComma = rawLines[i + 2].IndexOf(',', firstComma + 1);

                    string datestring = rawLines[i + 2].Substring(firstComma + 2, secondComma + 4 - firstComma);
                    newCheckin.date = DateTime.Parse(datestring);


                    string timestring = rawLines[i + 2].Substring(secondComma + 7);
                    int firstColon = timestring.IndexOf(':');
                    int secondColon = timestring.IndexOf(':', firstColon + 1);
                    TimeSpan timespan = TimeSpan.Parse(timestring.Substring(0, secondColon));
                    if (timestring.EndsWith("pm", true, System.Globalization.CultureInfo.InvariantCulture))
                    {
                        timespan += new TimeSpan(12, 0, 0);
                        //timespan.Add(new TimeSpan(12,0,0));
                    }

                    newCheckin.date += timespan;

                    checkins.Add(newCheckin);
                }
            }
        }

        public void Go()
        {
            monthSummaries = new List<MonthSummary>();
            weekSummary = new WeekSummary();
            checkins = new List<Checkin>();

            Font font = new Font("dejavusanscondensed_bold.ttf");

            uint monitorWidth = VideoMode.DesktopMode.Width - 64;
            uint monitorHeight = VideoMode.DesktopMode.Height - 128;

            Styles windowStyle = Styles.Titlebar | Styles.Close;

            app = new RenderWindow(new VideoMode(monitorWidth, monitorHeight, 32),
                "Blendo SVN vis", windowStyle);
            app.Clear(Color.White);
            app.Display();
            app.Closed += new EventHandler(app_Closed);

            OpenFileDialog fbd = new OpenFileDialog();
            fbd.InitialDirectory = Application.StartupPath;
            fbd.Filter = "SVN logs (.txt)|*.txt|All Files (*.*)|*.*";

            if (fbd.ShowDialog() == DialogResult.OK)
            {
            }
            else
            {
                app.Close();
                return;
            }

            


            //Read the log file.
            ParseCheckins(fbd.FileName);
            string filename = Path.GetFileNameWithoutExtension(fbd.FileName);

            if (checkins.Count <= 0)
            {
                MessageBox.Show(string.Format("Couldn't find SVN log data in file:\n{0}", fbd.FileName));
                return;
            }


            //Parse the month data.
            for (int i = 0; i < checkins.Count; i++)
            {
                int month = checkins[i].date.Month;
                int year = checkins[i].date.Year;

                MonthSummary curMonth = GetMonthSummary(month, year);

                if (curMonth != null)
                {
                    curMonth.count++;
                }
                else
                {
                    MonthSummary newMonth = new MonthSummary();
                    newMonth.month = month;
                    newMonth.year = year;

                    monthSummaries.Add(newMonth);
                }
            }

            int highestMonthCount = -1;
            for (int i = 0; i < monthSummaries.Count; i++)
            {
                if (monthSummaries[i].count > highestMonthCount)
                {
                    highestMonthCount = monthSummaries[i].count;
                }
            }

            float monthchunkHeight = (monitorHeight - 80) / (float)highestMonthCount;

            int lastYear = 0;
            for (int i = 0; i < monthSummaries.Count; i++)
            {
                bool showYear = false;
                if (monthSummaries[i].year != lastYear)
                {
                    showYear = true;
                    lastYear = monthSummaries[i].year;
                }

                monthSummaries[i].Initialize(i, monitorWidth, monitorHeight, font, showYear, monthchunkHeight);
            }

            //parse the week summary.
            weekSummary.Initialize(monitorWidth, monitorHeight, font, checkins);



            hourlySummary = new HourlySummary(checkins, font, monitorWidth);
            bool spaceDown = false;




            helptext = new Text(string.Format("blendo svn vis: {0}\nspacebar = change view.", filename), font);
            helptext.Position = new Vector2f(10, 10);
            helptext.Color = new Color(160, 160, 160);
            helptext.CharacterSize = 12;

            while (app.IsOpen())
            {
                app.DispatchEvents();
                app.Clear(Color.White);

                if (Keyboard.IsKeyPressed(Keyboard.Key.Space) && !spaceDown)
                {
                    spaceDown = true;
                    drawmode++;

                    if (drawmode > 2)
                    {
                        drawmode = 0;
                    }
                }
                if (!Keyboard.IsKeyPressed(Keyboard.Key.Space) && spaceDown)
                {
                    spaceDown = false;
                }


                if (Keyboard.IsKeyPressed(Keyboard.Key.Escape))
                {
                    app.Close();
                }


                for (int i = 0; i < monthSummaries.Count; i++)
                {
                    float speed = 0.2f;

                    if (Keyboard.IsKeyPressed(Keyboard.Key.RShift) || Keyboard.IsKeyPressed(Keyboard.Key.LShift))
                    {
                        speed *= 20;
                    }

                    if (Keyboard.IsKeyPressed(Keyboard.Key.Left))
                        monthSummaries[i].Reposition(speed);
                    else if (Keyboard.IsKeyPressed(Keyboard.Key.Right))
                        monthSummaries[i].Reposition(-speed);

                    if (Keyboard.IsKeyPressed(Keyboard.Key.Home) || Keyboard.IsKeyPressed(Keyboard.Key.Back))
                        monthSummaries[i].Reset();
                }


                //DRAW.

                app.Draw(helptext);


                if (drawmode == 0)
                {

                    for (int i = 0; i < monthSummaries.Count; i++)
                    {
                        monthSummaries[i].Draw(app);
                    }
                }
                else if (drawmode == 1)
                {

                    hourlySummary.Draw(app);
                }
                else
                {
                    weekSummary.Draw(app);
                }

                
                app.Display();
            }
        }

        MonthSummary GetMonthSummary(int month, int year)
        {
            for (int i = 0; i < monthSummaries.Count; i++)
            {
                if (monthSummaries[i].month == month && monthSummaries[i].year == year)
                {
                    return monthSummaries[i];
                }
            }

            return null;
        }

        void app_Closed(object sender, EventArgs e)
        {
            app.Close();
        }

        private string GetFileContents(string filepath)
        {
            string output = "";

            if (!File.Exists(filepath))
            {
                MessageBox.Show( string.Format("file not found:\n{0}", filepath));
                return null;
            }

            try
            {
                using (FileStream stream = File.Open(filepath, FileMode.Open, FileAccess.Read))
                {
                    using (StreamReader reader = new StreamReader(stream))
                    {
                        //dump file contents into a string.
                        output = reader.ReadToEnd();
                    }
                }
            }
            catch
            {
                MessageBox.Show(string.Format("There was a problem reading the file:\n{0}", filepath));
                return null;
            }

            return output;
        }
    }
}