﻿using System;
using System.Collections.Generic;

using SFML.Graphics;
using SFML.Window;

namespace blendosvnvis
{
    class WeekDay
    {
        public string day;
        public int count = 1;
        RectangleShape shape;

        Text dayText;
        Text countText;

        public WeekDay(string name)
        {
            this.day = name;
        }

        public void Initialize(uint monitorheight, int index, Font font, float chunkHeight)
        {
            const int BARWIDTH = 96;

            shape = new RectangleShape(new Vector2f(BARWIDTH, count * chunkHeight));
            shape.Origin = new Vector2f(0, count * chunkHeight);
            shape.FillColor = new Color(255, 120, 0);
            shape.Position = new Vector2f(30 + (index * (BARWIDTH + 2)), monitorheight - 12);

            dayText = new Text(day, font);
            dayText.Position = shape.Position + new Vector2f(2, -25);
            dayText.CharacterSize = 15;
            dayText.Color = Color.White;

            countText = new Text(count.ToString(), font);
            countText.Color = new Color(255, 120, 0);
            countText.CharacterSize = 24;
            countText.Position = shape.Position + new Vector2f(0, -shape.Size.Y - 28);
        }

        public void Draw(RenderWindow app)
        {
            app.Draw(shape);
            app.Draw(dayText);
            app.Draw(countText);
        }
    }

    class WeekSummary
    {
        WeekDay[] weekdays;

        public void Initialize(uint monitorWidth, uint monitorHeight, Font font, List<Checkin> checkins)
        {
            //set up the days.
            weekdays = new WeekDay[7]
            {
                new WeekDay("Sunday"),
                new WeekDay("Monday"),
                new WeekDay("Tuesday"),
                new WeekDay("Wednesday"),
                new WeekDay("Thursday"),
                new WeekDay("Friday"),
                new WeekDay("Saturday"),                
            };

            for (int i = 0; i < checkins.Count; i++)
            {
                for (int k = 0; k < weekdays.Length; k++)
                {
                    if (weekdays[k].day.StartsWith(checkins[i].date.DayOfWeek.ToString(), true, System.Globalization.CultureInfo.InvariantCulture))
                    {
                        weekdays[k].count++;
                    }
                }
            }

            int highestCount = -1;
            for (int k = 0; k < weekdays.Length; k++)
            {
                if (weekdays[k].count > highestCount)
                {
                    highestCount = weekdays[k].count;
                }
            }

            float chunkHeight = (monitorHeight - 75) / (float)highestCount;

            for (int k = 0; k < weekdays.Length; k++)
            {
                weekdays[k].Initialize(monitorHeight, k, font, chunkHeight);
            }
        }


        public void Draw(RenderWindow app)
        {
            for (int i = 0; i < weekdays.Length; i++)
            {
                weekdays[i].Draw(app);
            }
        }
    }
}
